/**
 * @license
 * Copyright (C) 2016-present Chi Vinh Le and contributors.
 *
 * This software may be modified and distributed under the terms
 * of the MIT license. See the LICENSE file for details.
 */
"use strict";
var blueprint_1 = require("./blueprint");
var createSimple = function (kind, callback) {
    return ({ instanceCallbacks: [{ kind: kind, callback: function (props) { return function () { return callback(props); }; } }] });
};
var createComparing = function (kind, callback) {
    return ({
        instanceCallbacks: function () {
            var prevProps;
            return [
                blueprint_1.propsCallback(function (props) {
                    if (prevProps === undefined) {
                        prevProps = props;
                    }
                    return props;
                }),
                {
                    kind: kind, callback: function (props) { return function () {
                        var prevPropsTmp = prevProps;
                        prevProps = props;
                        var result = callback(prevPropsTmp, props);
                        return result;
                    }; },
                },
            ];
        },
    });
};
function onWillMount(callback) {
    return createSimple("componentWillMountCallback", callback);
}
exports.onWillMount = onWillMount;
function onDidMount(callback) {
    return createSimple("componentDidMountCallback", callback);
}
exports.onDidMount = onDidMount;
function onWillUnmount(callback) {
    return createSimple("componentWillUnmountCallback", callback);
}
exports.onWillUnmount = onWillUnmount;
function onWillReceiveProps(callback) {
    return createComparing("componentWillReceivePropsCallback", callback);
}
exports.onWillReceiveProps = onWillReceiveProps;
function onWillUpdate(callback) {
    return createComparing("componentWillUpdateCallback", callback);
}
exports.onWillUpdate = onWillUpdate;
function onDidUpdate(callback) {
    return createComparing("componentDidUpdateCallback", callback);
}
exports.onDidUpdate = onDidUpdate;
function shouldUpdate(callback) {
    return createComparing("shouldComponentUpdateCallback", callback);
}
exports.shouldUpdate = shouldUpdate;

//# sourceMappingURL=lifecycle.js.map
