/**
 * @license
 * Copyright (C) 2016-present Chi Vinh Le and contributors.
 *
 * This software may be modified and distributed under the terms
 * of the MIT license. See the LICENSE file for details.
 */
import { propsCallback } from "./blueprint";
var createSimple = function (kind, callback) {
    return ({ instanceCallbacks: [{ kind: kind, callback: function (props) { return function () { return callback(props); }; } }] });
};
var createComparing = function (kind, callback) {
    return ({
        instanceCallbacks: function () {
            var prevProps;
            return [
                propsCallback(function (props) {
                    if (prevProps === undefined) {
                        prevProps = props;
                    }
                    return props;
                }),
                {
                    kind: kind, callback: function (props) { return function () {
                        var prevPropsTmp = prevProps;
                        prevProps = props;
                        var result = callback(prevPropsTmp, props);
                        return result;
                    }; },
                },
            ];
        },
    });
};
export function onWillMount(callback) {
    return createSimple("componentWillMountCallback", callback);
}
export function onDidMount(callback) {
    return createSimple("componentDidMountCallback", callback);
}
export function onWillUnmount(callback) {
    return createSimple("componentWillUnmountCallback", callback);
}
export function onWillReceiveProps(callback) {
    return createComparing("componentWillReceivePropsCallback", callback);
}
export function onWillUpdate(callback) {
    return createComparing("componentWillUpdateCallback", callback);
}
export function onDidUpdate(callback) {
    return createComparing("componentDidUpdateCallback", callback);
}
export function shouldUpdate(callback) {
    return createComparing("shouldComponentUpdateCallback", callback);
}

//# sourceMappingURL=lifecycle.js.map
